package com.muyu.cloud.market.controller;

import com.muyu.cloud.market.domin.Product;
import com.muyu.cloud.market.domin.req.ProductAddReq;
import com.muyu.cloud.market.domin.req.ProductListReq;
import com.muyu.cloud.market.domin.req.ProductUpdReq;
import com.muyu.cloud.market.domin.resp.ChosehowpayResp;
import com.muyu.cloud.market.domin.resp.ProductListResp;
import com.muyu.cloud.market.domin.resp.ProductTotalListResp;
import com.muyu.cloud.market.service.ProductApiService;
import com.muyu.common.core.domain.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @Author：weiran
 * @Package：com.muyu.cloud.market.controller
 * @Project：cloud-market
 * @name：ProductApiController
 * @Date：2024/8/20 14:43
 */
@Log4j2
@RestController
@RequestMapping("/product")
@Tag(name = "产品接口控制层",description = "进行产品接口管理、查看等相关操作")
public class ProductApiController {

    @Autowired
    private ProductApiService productApiService;

    /**
     * 查询全部产品接口
     * @param productListReq
     * @return
     */
    @PostMapping(path ="/list")
    @Operation(summary = "产品列表",description = "查询全部产品信息")
    public Result<ProductTotalListResp> selectList(@Validated @RequestBody ProductListReq productListReq){
        return Result.success(productApiService.selectList(productListReq));
    }


    /**
     * 添加产品接口
     * @param productAddReq
     * @return
     */
    @PostMapping
    @Operation(summary = "接口信息添加", description = "添加接口信息,添加成功才可以使用")
    public Result<String> save(@Validated @RequestBody ProductAddReq productAddReq){
        productApiService.save(Product.addProductList(productAddReq));
        return Result.success(null,"操作成功");
    }


    /**
     * 修改产品接口
     * @param productId
     * @param productUpdReq
     * @return
     */
    @PutMapping("/{productId}")
    @Operation(summary = "接口信息修改", description = "通过ID修改接口信息")
    public Result<String> update(
            @Schema(title = "产品ID", type = "Integer", defaultValue = "1", description = "修改产品信息需要依据的唯一条件")
            @PathVariable("productId") Integer productId,
            @RequestBody @Validated ProductUpdReq productUpdReq) {
        productApiService.updateById(Product.updProductList(productUpdReq,()->productId));
        return Result.success(null, "操作成功");
    }




    /**
     * 删除产品接口
     * @param productId
     * @return
     */
    @DeleteMapping("/{productId}")
    @Operation(summary = "接口信息删除",description = "通过Id删除接口信息")
    public Result<String> delete(@PathVariable("productId") Integer productId){
        productApiService.removeById(productId);
        return Result.success(null, "操作成功");
    }


    /**
     * 热点产品查询
     * @return
     */
    @RequestMapping(path ="/hotspotlist",method = RequestMethod.GET)
    @Operation(summary = "热点产品查询",description = "查询销量排名前五的产品")
    public Result<List<ProductListResp>> findhotspotproduct(){
        return Result.success(productApiService.selecthostportList());
    }

    /**
     * 产品类型查询
     * @return
     */
    @PostMapping(path = "/typeList")
    @Operation(summary = "产品类型",description = "查询全部产品类型")
    public Result productTypeList(){
        return Result.success(productApiService.selectproductTypeList());
    }


    /**
     * 产品的支付规格及对应的价格
     * @return
     */
    @PostMapping("/specification")
    @Operation(summary = "产品规格",description = "查询产品的支付规格及对应的价格")
    public Result selectspecification(@Validated @RequestBody ChosehowpayResp chosehowpayResp){
        return Result.success(productApiService.selectspecification(chosehowpayResp));
    }


    /**
     * 判断接口是否已被购买
     * @param product
     * @return
     */
    @PostMapping("/ifbuy")
    @Operation(summary = "是否购买",description = "判断是否已经购买")
    public Result ifbuy(@Validated @RequestBody Product product){
        String selectremark = productApiService.selectremark(product);
        if (selectremark.equals("否")){
            return Result.error("尊敬的客户，由于您没有对此产品进行购买，我们会给您提供两次的测试机会，若您想要多次使用本产品，请先购买！");
        }
        return Result.success();
    }


    /**
     * 测试次数-1
     * @param product
     * @return
     */
    @PostMapping("/reducetestcount")
    @Operation(summary = "测试次数-1",description = "测试次数-1")
    public void reducetestcount(@Validated @RequestBody Product product){
        Integer reducetestcount = productApiService.reducetestcount(product);
    }

    /**
     * 判断测试次数
     * @param product
     * @return
     */
    @PostMapping("/iftest")
    @Operation(summary = "判断测试次数是否<=0",description = "判断测试次数是否<=0")
    public Result iftest(@Validated @RequestBody Product product){
        Integer iftest = productApiService.iftest(product);
        return Result.success(iftest);
    }
}
